import type { DocsJson, RenderItem } from "./json";
export declare const kernels: {
    [key: string]: unknown;
};
export declare function embed_items_notebook(docs_json: DocsJson, render_items: RenderItem[]): Promise<void>;
//# sourceMappingURL=notebook.d.ts.map